/**
 * Ext GWT 3.0.0-beta4 - Ext for GWT
 * Copyright(c) 2007-2011, Sencha, Inc.
 * licensing@sencha.com
 *
 * http://sencha.com/license
 */
package com.sencha.gxt.desktop.client.browser;

import com.google.gwt.user.client.ui.HasWidgets;
import com.google.web.bindery.event.shared.HandlerRegistration;
import com.sencha.gxt.desktop.client.desktopapp.DesktopBus;
import com.sencha.gxt.desktop.client.event.RemoveFileModelEvent;
import com.sencha.gxt.desktop.client.event.RemoveFileModelEvent.RemoveFileModelHandler;
import com.sencha.gxt.desktop.client.event.UpdateFileModelEvent;
import com.sencha.gxt.desktop.client.event.UpdateFileModelEvent.UpdateFileModelHandler;
import com.sencha.gxt.desktop.client.persistence.FileModel;
import com.sencha.gxt.desktop.client.persistence.FileSystem;

public class BrowserPresenterImpl implements BrowserPresenter {

  private DesktopBus desktopBus;
  private FileSystem fileSystem;
  private FileModel fileModel;

  private BrowserView browserView;
  private HandlerRegistration updateFileModelRegistration;
  private UpdateFileModelHandler updateFileModelHandler;
  private HandlerRegistration removeFileModelRegistration;
  private RemoveFileModelHandler removeFileModelHandler;

  public BrowserPresenterImpl(DesktopBus desktopBus, FileSystem fileSystem, FileModel fileModel) {
    this.desktopBus = desktopBus;
    this.fileSystem = fileSystem;
    this.fileModel = fileModel;
  }

  @Override
  public void bind() {
    getUpdateFileModelRegistration();
    getRemoveFileModelRegistration();
    updateTitle();
    updateUrl();
  }

  @Override
  public void go(HasWidgets hasWidgets) {
    hasWidgets.add(getBrowserView().asWidget());
  }

  @Override
  public void onClose() {
    getBrowserView().close();
  }

  @Override
  public void onSave() {
    // menu item exists but currently no action
  }

  @Override
  public void unbind() {
    getUpdateFileModelRegistration().removeHandler();
    getRemoveFileModelRegistration().removeHandler();
  }

  private BrowserView getBrowserView() {
    if (browserView == null) {
      browserView = new BrowserViewImpl(this);
    }
    return browserView;
  }

  private DesktopBus getDesktopBus() {
    return desktopBus;
  }

  private FileModel getFileModel() {
    return fileModel;
  }

  private FileSystem getFileSystem() {
    return fileSystem;
  }

  private RemoveFileModelHandler getRemoveFileModelHandler() {
    if (removeFileModelHandler == null) {
      removeFileModelHandler = new RemoveFileModelHandler() {
        @Override
        public void onRemoveFileModel(RemoveFileModelEvent removeFileModelEvent) {
          onRemoveFileModelEvent(removeFileModelEvent);
        }
      };
    }
    return removeFileModelHandler;
  }

  private HandlerRegistration getRemoveFileModelRegistration() {
    if (removeFileModelRegistration == null) {
      removeFileModelRegistration = getDesktopBus().addRemoveFileModelHandler(getRemoveFileModelHandler());
    }
    return removeFileModelRegistration;
  }

  private UpdateFileModelHandler getUpdateFileModelHandler() {
    if (updateFileModelHandler == null) {
      updateFileModelHandler = new UpdateFileModelHandler() {
        @Override
        public void onUpdateFileModel(UpdateFileModelEvent updateFileModelEvent) {
          onUpdateFileModelEvent(updateFileModelEvent);
        }
      };
    }
    return updateFileModelHandler;
  }

  private HandlerRegistration getUpdateFileModelRegistration() {
    if (updateFileModelRegistration == null) {
      updateFileModelRegistration = getDesktopBus().addUpdateFileModelHandler(getUpdateFileModelHandler());
    }
    return updateFileModelRegistration;
  }

  private void onRemoveFileModelEvent(RemoveFileModelEvent removeFileModelEvent) {
    FileModel fileModel = removeFileModelEvent.getFileModel();
    if (fileModel == getFileModel()) {
      getBrowserView().close();
    }
  }

  private void onUpdateFileModelEvent(UpdateFileModelEvent updateFileModelEvent) {
    updateTitle();
    FileModel fileModel = updateFileModelEvent.getFileModel();
    if (fileModel == getFileModel()) {
      updateUrl();
    }
  }

  private void updateTitle() {
    String newTitle = "Browser - " + getFileSystem().getPath(fileModel);
    getBrowserView().setTitle(newTitle);
  }

  private void updateUrl() {
    getBrowserView().setUrl(getFileModel().getName());
  }

}
