/**
 * Ext GWT 3.0.0-beta4 - Ext for GWT
 * Copyright(c) 2007-2011, Sencha, Inc.
 * licensing@sencha.com
 *
 * http://sencha.com/license
 */
package com.sencha.gxt.desktop.client.browser;

import com.google.gwt.event.logical.shared.SelectionEvent;
import com.google.gwt.event.logical.shared.SelectionHandler;
import com.google.gwt.user.client.ui.Frame;
import com.google.gwt.user.client.ui.Widget;
import com.sencha.gxt.desktop.client.filemanager.images.Images;
import com.sencha.gxt.widget.core.client.Window;
import com.sencha.gxt.widget.core.client.container.VerticalLayoutContainer;
import com.sencha.gxt.widget.core.client.container.VerticalLayoutContainer.VerticalLayoutData;
import com.sencha.gxt.widget.core.client.event.HideEvent;
import com.sencha.gxt.widget.core.client.event.HideEvent.HideHandler;
import com.sencha.gxt.widget.core.client.event.ShowEvent;
import com.sencha.gxt.widget.core.client.event.ShowEvent.ShowHandler;
import com.sencha.gxt.widget.core.client.menu.Menu;
import com.sencha.gxt.widget.core.client.menu.MenuBar;
import com.sencha.gxt.widget.core.client.menu.MenuBarItem;
import com.sencha.gxt.widget.core.client.menu.MenuItem;

public class BrowserViewImpl implements BrowserView, HideHandler, ShowHandler {

  private BrowserPresenter browserPresenter;
  private Window window;
  private VerticalLayoutContainer verticalLayoutContainer;
  private Frame frame;
  private MenuBar menuBar;
  private MenuBarItem fileMenuBarItem;
  private Menu fileMenu;
  private MenuItem saveMenuItem;
  private MenuItem closeMenuItem;
  private SelectionHandler<MenuItem> saveSelectionHandler;
  private SelectionHandler<MenuItem> closeSelectionHandler;

  public BrowserViewImpl(BrowserPresenter browserPresenter) {
    this.browserPresenter = browserPresenter;
  }

  @Override
  public Widget asWidget() {
    return getWindow();
  }

  @Override
  public void close() {
    getWindow().hide();
  }

  @Override
  public void onHide(HideEvent event) {
    getBrowserPresenter().unbind();
  }

  @Override
  public void onShow(ShowEvent event) {
    getBrowserPresenter().bind();
  }

  @Override
  public void setTitle(String title) {
    getWindow().setHeadingText(title);
  }

  @Override
  public void setUrl(String url) {
    getFrame().setUrl(url);
  }

  private BrowserPresenter getBrowserPresenter() {
    return browserPresenter;
  }

  private MenuItem getCloseMenuItem() {
    if (closeMenuItem == null) {
      closeMenuItem = new MenuItem("Close", getCloseSelectionHandler());
      closeMenuItem.setIcon(Images.getImageResources().cross());
    }
    return closeMenuItem;
  }

  private SelectionHandler<MenuItem> getCloseSelectionHandler() {
    if (closeSelectionHandler == null) {
      closeSelectionHandler = new SelectionHandler<MenuItem>() {
        @Override
        public void onSelection(SelectionEvent<MenuItem> event) {
          browserPresenter.onClose();
        }
      };
    }
    return closeSelectionHandler;
  }

  private Menu getFileMenu() {
    if (fileMenu == null) {
      fileMenu = new Menu();
      fileMenu.add(getSaveMenuItem());
      fileMenu.add(getCloseMenuItem());
    }
    return fileMenu;
  }

  private MenuBarItem getFileMenuBarItem() {
    if (fileMenuBarItem == null) {
      fileMenuBarItem = new MenuBarItem("File");
      fileMenuBarItem.setMenu(getFileMenu());
    }
    return fileMenuBarItem;
  }

  private Frame getFrame() {
    if (frame == null) {
      frame = new Frame();
    }
    return frame;
  }

  private MenuBar getMenuBar() {
    if (menuBar == null) {
      menuBar = new MenuBar();
      menuBar.add(getFileMenuBarItem());
    }
    return menuBar;
  }

  private MenuItem getSaveMenuItem() {
    if (saveMenuItem == null) {
      saveMenuItem = new MenuItem("Save", getSaveSelectionHandler());
      saveMenuItem.setIcon(Images.getImageResources().disk());
    }
    return saveMenuItem;
  }

  private SelectionHandler<MenuItem> getSaveSelectionHandler() {
    if (saveSelectionHandler == null) {
      saveSelectionHandler = new SelectionHandler<MenuItem>() {
        @Override
        public void onSelection(SelectionEvent<MenuItem> event) {
          browserPresenter.onSave();
        }
      };
    }
    return saveSelectionHandler;
  }

  private VerticalLayoutContainer getVerticalLayoutContainer() {
    if (verticalLayoutContainer == null) {
      verticalLayoutContainer = new VerticalLayoutContainer();
      verticalLayoutContainer.add(getMenuBar(), new VerticalLayoutData(1, -1));
      verticalLayoutContainer.add(getFrame(), new VerticalLayoutData(1, 1));
    }
    return verticalLayoutContainer;
  }

  private Window getWindow() {
    if (window == null) {
      window = new Window();
      window.getHeader().setIcon(Images.getImageResources().page_white());
      window.setMinimizable(true);
      window.setMaximizable(true);
      window.setOnEsc(false);
      window.setPixelSize(500, 400);
      window.addHideHandler(this);
      window.addShowHandler(this);
      window.add(getVerticalLayoutContainer());
    }
    return window;
  }

}
