/**
 * Ext GWT 3.0.0-beta4 - Ext for GWT
 * Copyright(c) 2007-2011, Sencha, Inc.
 * licensing@sencha.com
 *
 * http://sencha.com/license
 */
package com.sencha.gxt.desktop.client.wordprocessor;

import com.google.gwt.event.logical.shared.SelectionEvent;
import com.google.gwt.event.logical.shared.SelectionHandler;
import com.google.gwt.user.client.ui.Widget;
import com.sencha.gxt.desktop.client.filemanager.images.Images;
import com.sencha.gxt.widget.core.client.Window;
import com.sencha.gxt.widget.core.client.container.VerticalLayoutContainer;
import com.sencha.gxt.widget.core.client.container.VerticalLayoutContainer.VerticalLayoutData;
import com.sencha.gxt.widget.core.client.event.HideEvent;
import com.sencha.gxt.widget.core.client.event.HideEvent.HideHandler;
import com.sencha.gxt.widget.core.client.event.ShowEvent;
import com.sencha.gxt.widget.core.client.event.ShowEvent.ShowHandler;
import com.sencha.gxt.widget.core.client.form.HtmlEditor;
import com.sencha.gxt.widget.core.client.menu.Menu;
import com.sencha.gxt.widget.core.client.menu.MenuBar;
import com.sencha.gxt.widget.core.client.menu.MenuBarItem;
import com.sencha.gxt.widget.core.client.menu.MenuItem;

public class WordProcessorViewImpl implements WordProcessorView, HideHandler, ShowHandler {

  private WordProcessorPresenter wordProcessorPresenter;
  private Window window;
  private VerticalLayoutContainer verticalLayoutContainer;
  private HtmlEditor htmlEditor;
  private MenuBar menuBar;
  private MenuBarItem fileMenuBarItem;
  private Menu fileMenu;
  private MenuItem saveMenuItem;
  private MenuItem closeMenuItem;
  private SelectionHandler<MenuItem> saveSelectionHandler;
  private SelectionHandler<MenuItem> closeSelectionHandler;

  public WordProcessorViewImpl(WordProcessorPresenter wordProcessorPresenter) {
    this.wordProcessorPresenter = wordProcessorPresenter;
  }

  @Override
  public Widget asWidget() {
    return getWindow();
  }

  @Override
  public void close() {
    getWindow().hide();
  }

  @Override
  public String getValue() {
    return getHtmlEditor().getValue();
  }

  @Override
  public void onHide(HideEvent event) {
    getWordProcessorPresenter().unbind();
  }

  @Override
  public void onShow(ShowEvent event) {
    getWordProcessorPresenter().bind();
    getHtmlEditor().focus();
  }

  @Override
  public void setTitle(String title) {
    getWindow().setHeadingText(title);
  }

  @Override
  public void setValue(String value) {
    getHtmlEditor().setValue(value);
  }

  private MenuItem getCloseMenuItem() {
    if (closeMenuItem == null) {
      closeMenuItem = new MenuItem("Close", getCloseSelectionHandler());
      closeMenuItem.setIcon(Images.getImageResources().cross());
    }
    return closeMenuItem;
  }

  private SelectionHandler<MenuItem> getCloseSelectionHandler() {
    if (closeSelectionHandler == null) {
      closeSelectionHandler = new SelectionHandler<MenuItem>() {
        @Override
        public void onSelection(SelectionEvent<MenuItem> event) {
          wordProcessorPresenter.onClose();
        }
      };
    }
    return closeSelectionHandler;
  }

  private Menu getFileMenu() {
    if (fileMenu == null) {
      fileMenu = new Menu();
      fileMenu.add(getSaveMenuItem());
      fileMenu.add(getCloseMenuItem());
    }
    return fileMenu;
  }

  private MenuBarItem getFileMenuBarItem() {
    if (fileMenuBarItem == null) {
      fileMenuBarItem = new MenuBarItem("File");
      fileMenuBarItem.setMenu(getFileMenu());
    }
    return fileMenuBarItem;
  }

  private HtmlEditor getHtmlEditor() {
    if (htmlEditor == null) {
      htmlEditor = new HtmlEditor();
    }
    return htmlEditor;
  }

  private MenuBar getMenuBar() {
    if (menuBar == null) {
      menuBar = new MenuBar();
      menuBar.add(getFileMenuBarItem());
    }
    return menuBar;
  }

  private MenuItem getSaveMenuItem() {
    if (saveMenuItem == null) {
      saveMenuItem = new MenuItem("Save", getSaveSelectionHandler());
      saveMenuItem.setIcon(Images.getImageResources().disk());
    }
    return saveMenuItem;
  }

  private SelectionHandler<MenuItem> getSaveSelectionHandler() {
    if (saveSelectionHandler == null) {
      saveSelectionHandler = new SelectionHandler<MenuItem>() {
        @Override
        public void onSelection(SelectionEvent<MenuItem> event) {
          wordProcessorPresenter.onSave();
        }
      };
    }
    return saveSelectionHandler;
  }

  private VerticalLayoutContainer getVerticalLayoutContainer() {
    if (verticalLayoutContainer == null) {
      verticalLayoutContainer = new VerticalLayoutContainer();
      verticalLayoutContainer.add(getMenuBar(), new VerticalLayoutData(1, -1));
      verticalLayoutContainer.add(getHtmlEditor(), new VerticalLayoutData(1, 1));
    }
    return verticalLayoutContainer;
  }

  private Window getWindow() {
    if (window == null) {
      window = new Window() {
        {
          removeFromParentOnHide = false;
        }
      };
      window.getHeader().setIcon(Images.getImageResources().page_white());
      window.setMinimizable(true);
      window.setMaximizable(true);
      window.setPixelSize(500, 400);
      window.setOnEsc(false);
      window.addHideHandler(this);
      window.addShowHandler(this);
      window.add(getVerticalLayoutContainer());
    }
    return window;
  }

  private WordProcessorPresenter getWordProcessorPresenter() {
    return wordProcessorPresenter;
  }

}
